from .models.ptymodels import (
    PCard,
    PMember,
    PEvent,
    PCostume,
    PItem,
    PAreaItem,
    PComic,
    PBackground,
    PStamp,
    PTitle,
    PInterface,
    POfficialArt
    )
from .loader import BandoriLoader


class PBandoriApi(BandoriLoader):
    '''
    Represents a class that interacts with the bandori party API
    '''

    def __init__(self, region='en/', party=True):
        super().__init__(region)
        self.party = party

    def get_cards(self, id: list = [], filters: dict = {}) -> list:
        '''
        Get card by ids, as Card objects.
        If the list is empty, will get all cards.
        '''
        d = self._api_get(id=id, url=self.URL_PARTY+'cards/',
                          filters=filters)

        return [PCard(data) for data in d]

    def get_members(self, id: list = [], filters: dict = {}) -> list:
        '''
        Get member by ids, as Member objects.
        If the list is empty, will get all members.
        '''
        d = self._api_get(id=id, url=self.URL_PARTY+'members/',
                          filters=filters)

        return [PMember(data) for data in d]

    def get_events(self, id: list = [], filters: dict = {}) -> list:
        '''
        Get events by ids, as Event objects.
        If the list is empty, will get all events.
        '''
        if not id:
            if not filters:
                events = self._full_event_loader(url=self.URL_PARTY+'events/',
                                                 filters=filters)
            else:
                events = self._full_event_loader(url=self.URL_PARTY+'events/',
                                                 filters={})
                events = [e for e in events if self._check_filters(
                          filters=filters, obj=e)]
        else:
            events = self._api_get(id=id, url=self.URL_PARTY+'events/',
                                   filters=filters)
            for i, event in enumerate(events):
                event['id'] = id[i]

        return [PEvent(event) for event in events]

    def get_current_event(self) -> dict:
        '''
        Returns the current ongoing event, as provided by bandori database.
        '''
        event = self._retrieve_response(self.URL_GA+'event/')
        id = event["eventId"] + 3  # offset of 3 to get the correct event

        return self.get_events(id=[id])[0]

    def get_costumes(self, id: list = [], filters: dict = {}) -> list:
        '''
        Get costume by ids, as Costume objects.
        If the list is empty all costumes will be returned.
        '''
        d = self._api_get(id=id, url=self.URL_PARTY+'costumes/',
                          filters=filters)

        return [PCostume(data) for data in d]

    def get_items(self, id: list = [], filters: dict = {}) -> list:
        '''
        Get item by ids, as Item objects.
        If the list is empty all items will be returned.
        '''
        d = self._api_get(id=id, url=self.URL_PARTY+'items/', filters=filters)

        return [PItem(data) for data in d]

    def get_areaitems(self, id: list = [], filters: dict = {}) -> list:
        '''
        Get areaitem by ids, as AreaItem objects.
        If the list is empty all items will be returned.
        '''
        d = self._api_get(id=id, url=self.URL_PARTY+'areaitems/',
                          filters=filters)

        return [PAreaItem(data) for data in d]

    def get_assets(self, id: list = [], filters={}) -> dict:
        '''
        Get bandori party asset by ids.
        If the list is empty all items will be returned.
        The return value is a dict with keys to the categories of assets,
        and for values a list of Asset objects.
        '''
        d = self._api_get(id=id, url=self.URL_PARTY+'assets/', filters=filters)

        sorted = {"comic": [], "background": [], "stamp": [], "title": [],
                  "interface": [], "officialart": []}
        for data in d:
            type = data["i_type"]
            if type == 'comic':
                sorted["comic"].append(PComic(data))
            elif type == 'background':
                sorted["background"].append(PBackground(data))
            elif type == 'stamp':
                sorted["stamp"].append(PStamp(data))
            elif type == 'title':
                sorted["title"].append(PTitle(data))
            elif type == 'interface':
                sorted["interface"].append(PInterface(data))
            else:
                sorted["officialart"].append(POfficialArt(data))

        return sorted



import requests


class BandoriLoader:
    '''
    Represents a class that makes api calls to bandori.party
    and bandori database.
    This only uses GET requests.
    '''
    def __init__(self, region='en/'):
        self.region = region
        self.URL_PARTY = "https://bandori.party/api/"
        self.URL_GA = "https://api.bandori.ga/v1/" + region
        self.URL_GA_RES = "https://res.bandori.ga"

    class FailedRequest(Exception):
        pass

    def _retrieve_response(self, url='') -> dict:
        '''
        Get a response from the url and returns the result
        as a dict.
        '''
        res = requests.get(url)
        if res.status_code != 200:
            raise BandoriLoader.FailedRequest(
                f'Could not get request from {url}')

        return res.json()

    def _retrieve_responses(self, url='', filters={}) -> list:
        '''
        ### FOR BANDORI.PARTY API

        Gets responses from provided url and returns the
        result as a list of dictionaries.
        This is intended to get all pages.
        '''
        res = []
        page = url

        while(True):
            response = requests.get(page)
            if response.status_code != 200:
                raise BandoriLoader.FailedRequest(
                    f'Could not get request from {page}. Stopping operation.')

            data = response.json()

            if data["next"] is None:
                for d in data["results"]:
                    if self._check_filters(filters=filters, obj=d):
                        res.append(d)
                break
            else:
                for d in data["results"]:
                    if self._check_filters(filters=filters, obj=d):
                        res.append(d)
                page = data["next"]

        return res

    def _check_filters(self, filters={}, obj: dict = None):
        '''
        Checks if the object (a dict) has the correct key-pair values
        corresponding to the filters.
        '''
        if not filters:
            return True

        for key, value in filters.items():
            if obj.get(key) != value:
                return False
        return True

    def _api_get(self, id: list = [], url='', party=True, filters={}):
        '''
        Handles getting responses from the APIs.
        The result may be returned as a list (of dicts) or a dict.
        '''
        if party:
            if not id:
                return self._retrieve_responses(url=url, filters=filters)
            else:
                res = []
                for i in id:
                    obj = self._retrieve_response(url + str(i))
                    if self._check_filters(obj=obj, filters=filters):
                        res.append(obj)

                return res
        else:
            if not id:
                d = self._retrieve_response(url)

                if type(d) == dict and d.get('data') is not None:
                    d = d['data']
                    res = []

                    for o in d:
                        if self._check_filters(obj=o, filters=filters):
                            res.append(o)

                    return res
                else:
                    res = []
                    for o in d:
                        if self._check_filters(obj=o, filters=filters):
                            res.append(o)
                    return res

            else:
                res = []
                for i in id:
                    obj = self._retrieve_response(url + str(i))
                    if self._check_filters(obj=obj, filters=filters):
                        res.append(obj)

                return res

    def _full_event_loader(self, url='', filters={}) -> list:
        '''
        ### FOR BANDORI.PARTY API

        A function that returns a list of dicts.

        Each dict is an event.

        It grabs all events from bandori.party api,
        and adds the appropriate id to the event.
        '''
        events = []
        id = 1  # assuming the smallest event id is 1.

        # getting the total count of events.
        data = self._retrieve_response(url)
        total_count = data["count"]

        # adding all events to the list.
        while len(events) < total_count:
            try:
                data = self._retrieve_response(url + str(id))
                if self._check_filters(filters=filters, obj=data):
                    if 'detail' in data.keys():
                        pass
                    else:
                        data['id'] = id
                        events.append(data)
                    id += 1

            except BandoriLoader.FailedRequest:
                id += 1
                continue

        return events



from ..loader import BandoriLoader
import json


class BandoriObject(BandoriLoader):
    '''
    Represents information retrieved from the api
    '''
    def __init__(self, data: dict, id_name='id', region='en/'):
        super().__init__(region)

        self.id = data.get(id_name)
        self.data = data

    def __lt__(self, other):
        return self.id < other.id

    def __str__(self):
        return str(self.data)

    def __iter__(self):
        return iter(self.data)

    def dump(self, file='dump.txt', mode='w'):
        with open(file, mode) as f:
            json.dump(self.data, f)



from .models.gamodels import (
    DCard,
    DSong,
    DGacha,
    DBand,
    DMember,
    DDegree,
    DComic,
    DStamp,
    DEvent
)
from .loader import BandoriLoader


class DBandoriApi(BandoriLoader):
    '''
    Represents a class that interacts with the bandori database API
    '''

    def __init__(self, region='en/'):
        super().__init__(region)
        self.party = False

    def get_cards(self, id: list = [], filters={}) -> list:
        '''
        Get card by ids, as Card objects.
        If the list is empty, will get all cards.
        '''
        d = self._api_get(id=id, url=self.URL_GA+'card/',
                          party=self.party, filters=filters)

        return [DCard(data, region=self.region) for data in d]

    def get_members(self, id: list = [], filters={}) -> list:
        '''
        Get member by ids, as Member objects.
        If the list is empty, will get all members.
        '''
        d = self._api_get(id=id, url=self.URL_GA+'chara/',
                          party=self.party, filters=filters)

        return [DMember(data) for data in d]

    def get_current_event(self) -> DEvent:
        '''
        Returns the current ongoing event, as provided by bandori database.
        '''
        event = self._retrieve_response(self.URL_GA+'event/')

        return DEvent(data=event, region=self.region)

    def get_bands(self, filters: dict = {}) -> list:
        '''
        Get all bands as a list of Band objects.
        This cannot search by id.
        '''
        d = self._api_get(id=[], url=self.URL_GA+'band/', party=False,
                          filters=filters)

        return [DBand(data, region=self.region) for data in d]

    def get_songs(self, id: list = [], filters: dict = {}) -> list:
        '''
        Get song by ids, as Song objects.

        If the list is empty all songs will be returned.
        '''
        d = self._api_get(id=id, url=self.URL_GA+'music/', party=False,
                          filters=filters)

        return [DSong(data, region=self.region) for data in d]

    def get_gachas(self, id: list = [], filters: dict = {}) -> list:
        '''
        Get gacha by ids, as Gacha objects.

        If the list is empty all gacha will be returned.
        '''
        d = self._api_get(id=id, url=self.URL_GA+'gacha/', party=False,
                          filters=filters)

        return [DGacha(data, region=self.region) for data in d]

    def get_active_gachas(self) -> list:
        '''
        Get active gachas as Gacha objects.
        '''
        d = self._api_get(url=self.URL_GA+'gacha/current', party=False)

        return [DGacha(data, region=self.region) for data in d]

    def get_rankings(self, id: list = [], filters: dict = {}) -> list:
        '''
        Get Degrees by ids, as Degree objects.

        If the list is empty all Degrees will be returned.
        '''
        d = self._api_get(id=id, url=self.URL_GA+'degree/', party=False,
                          filters=filters)

        return [DDegree(data, region=self.region) for data in d]

    def get_comics(self, id: list = [], filters: dict = {}) -> list:
        '''
        Get the loading screen komas from the bandori database api
        '''
        d = self._api_get(id=id, url=self.URL_GA+'sfc/', party=False,
                          filters=filters)

        return [DComic(data, region=self.region) for data in d]

    def get_stamps(self, id: list = [], filters: dict = {}) -> list:
        '''
        Get the stamps from the bandori database api
        '''
        d = self._api_get(id=id, url=self.URL_GA+'sfc/', party=False,
                          filters=filters)

        return [DStamp(data, region=self.region) for data in d]



from .p_api import PBandoriApi
from .d_api import DBandoriApi


def bandori_api(region: str = 'en/', party: bool = True):

    if party:
        return PBandoriApi(region=region, party=party)
    else:
        return DBandoriApi(region=region)



import datetime
from .base import BandoriObject


###############################################################################
# Bandori Party models

class PCard(BandoriObject):
    '''
    Represents a bang dream card.
    '''
    def __init__(self, data: dict):
        super().__init__(data)

        self.member = data.get("member")
        self.rarity = data.get("i_rarity")
        self.attribute = data.get("i_attribute")
        self.name = data.get("name")
        self.japanese_name = data.get("japanese_name")
        self.is_promo = data.get("is_promo")
        self.is_original = data.get("is_original")
        self.image = data.get("image")
        self.image_trained = data.get("image_trained")
        self.art = data.get("art")
        self.art_trained = data.get("art_trained")
        self.transparent = data.get("transparent")
        self.transparent_trained = data.get("transparent_trained")
        self.skill_name = data.get("skill_name")
        self.japanese_skill_name = data.get("japanese_skill_name")
        self.skill_type = data.get("i_skill_type")
        self.side_skill_type = data.get("i_side_skill_type")
        self.skill_template = data.get("skill_template")
        self.skill_variables = data.get("skill_variables")
        self.side_skill_template = data.get("side_skill_template")
        self.side_skill_variables = data.get("side_skill_variables")
        self.full_skill = data.get("full_skill")
        self.performance_min = data.get("performance_min")
        self.performance_max = data.get("performance_max")
        self.performance_trained_max = data.get("performance_trained_max")
        self.technique_min = data.get("technique_min")
        self.technique_max = data.get("technique_max")
        self.technique_trained_max = data.get("technique_trained_max")
        self.visual_min = data.get("visual_min")
        self.visual_max = data.get("visual_max")
        self.visual_trained_max = data.get("visual_trained_max")
        self.cameo = data.get("cameo_members")

    def get_card_member(self):

        d = self._api_get(id=[self.member], url=self.URL_PARTY+'members/')

        return PMember(d[0])

    def get_cameo_members(self):

        d = self._api_get(id=self.cameo, url=self.URL_PARTY+'members/')

        return [PMember(data) for data in d]


class PMember(BandoriObject):
    '''
    Represents a bang dream member.
    '''
    def __init__(self, data: dict):
        super().__init__(data)
        self.name = data.get("name")
        self.japanese_name = data.get("japanese_name")
        self.image = data.get("image")
        self.square_image = data.get("square_image")
        self.band = data.get("i_band")  # TODO: match band to Band object?
        self.school = data.get("school")
        self.year = data.get("i_school_year")
        self.romaji_cv = data.get("romaji_CV")
        self.cv = data.get("CV")
        self.birthday = data.get("birthday")
        self.food_likes = data.get("food_like")
        self.food_dislikes = data.get("food_dislike")
        self.astro = data.get("i_astrological_sign")
        self.instrument = data.get("instrument")
        self.description = data.get("description")


class PEvent(BandoriObject):
    '''
    Represents a bang dream game event.
    '''

    def __init__(self, data: dict):
        super().__init__(data)

        self.name = data.get("name")
        self.japanese_name = data.get("japanese_name")
        self.type = data.get("i_type")
        self.image = data.get("image")

        self.english_start_date = data.get("english_start_date")
        self.english_end_date = data.get("english_end_date")
        self.jp_start_date = data.get("start_date")
        self.jp_end_date = data.get("end_date")
        self.tw_start_date = data.get("taiwanese_start_date")
        self.tw_end_date = data.get("taiwanese_end_date")
        self.kr_start_date = data.get("korean_start_date")
        self.kr_end_date = data.get("korean_end_date")

        self.versions_available = data.get("c_versions")
        self.main_card = data.get("main_card")
        self.secondary_card = data.get("secondary_card")
        self.boost_attribute = data.get("i_boost_attribute")
        self.boost_stat = data.get("i_boost_stat")
        self.boost_members = data.get("boost_members")

    def get_start_date(self, region='en'):
        if region == 'en':
            if self.english_start_date is not None:
                return datetime.datetime.strptime(
                    self.english_start_date, '%Y-%m-%dT%H:%M:%SZ')
            else:
                return -1
        elif region == 'jp':
            if self.jp_start_date is not None:
                return datetime.datetime.strptime(
                    self.jp_start_date, '%Y-%m-%dT%H:%M:%SZ')
            else:
                return -1
        elif region == 'tw':
            if self.tw_start_date is not None:
                return datetime.datetime.strptime(
                    self.tw_start_date, '%Y-%m-%dT%H:%M:%SZ')
            else:
                return -1
        else:
            if self.kr_start_date is not None:
                return datetime.datetime.strptime(
                    self.kr_start_date, '%Y-%m-%dT%H:%M:%SZ')
            else:
                return -1

    def get_end_date(self, region='en'):
        if region == 'en':
            if self.english_end_date is not None:
                return datetime.datetime.strptime(
                    self.english_end_date, '%Y-%m-%dT%H:%M:%SZ')
            else:
                return -1
        elif region == 'jp':
            if self.jp_end_date is not None:
                return datetime.datetime.strptime(
                    self.jp_end_date, '%Y-%m-%dT%H:%M:%SZ')
            else:
                return -1
        elif region == 'tw':
            if self.tw_end_date is not None:
                return datetime.datetime.strptime(
                    self.tw_end_date, '%Y-%m-%dT%H:%M:%SZ')
            else:
                return -1
        else:
            if self.kr_end_date is not None:
                return datetime.datetime.strptime(
                    self.kr_end_date, '%Y-%m-%dT%H:%M:%SZ')
            else:
                return -1

    def get_main_card(self):

        data = self._api_get(id=[self.main_card],
                             url=self.URL_PARTY+'cards/')

        return PCard(data[0])

    def get_secondary_card(self):

        data = self._api_get(id=[self.secondary_card],
                             url=self.URL_PARTY+'cards/')

        return PCard(data[0])

    def get_boost_members(self):

        d = self._api_get(id=self.boost_attribute,
                          url=self.URL_PARTY+'members/')

        return [PMember(data) for data in d]


class PCostume(BandoriObject):
    '''
    Represents a bang dream costume.
    '''
    def __init__(self, data: dict):
        super().__init__(data)
        self.type = data.get("i_costume_type")
        self.card = data.get("card")
        self.member = data.get("member")
        self.name = data.get("name")

        self.display_image = data.get("display_image")

    def get_costume_member(self):

        data = self._api_get(id=[self.member], url=self.URL_PARTY+'members/')

        return PMember(data[0])

    def get_costume_card(self):

        d = self._api_get(id=[self.card], url=self.URL_PARTY+'cards/')

        return PCard(d[0])


class PItem(BandoriObject):
    '''
    Represents a bang dream in-game item
    '''
    def __init__(self, data: dict):
        super().__init__(data)
        self.name = data.get("name")
        self.type = data.get("i_type")
        self.description = data.get("m_description")
        self.image = data.get("image")


class PAreaItem(BandoriObject):
    '''
    Represents a bang dream area item
    '''
    def __init__(self, data):
        super().__init__(data)
        self.name = data.get("name")
        self.image = data.get("image")
        self.area = data.get("area")    # TODO: name of area??
        self.type = data.get("i_type")
        self.instrument = data.get("i_instrument")
        self.attribute = data.get("i_attribute")
        self.boost_stat = data.get("i_boost_stat")
        self.max_level = data.get("max_level")
        self.values = data.get("value_list")
        self.description = data.get("about")


class PAsset(BandoriObject):
    '''
    Represents a bang dream asset as defined by bandori.party

    Known assets:
    comic
    background
    stamp
    title
    interface
    officialart
    '''
    def __init__(self, data):
        super().__init__(data)
        self.type = data.get("i_type")


class PComic(PAsset):
    def __init__(self, data):
        super().__init__(data)
        self.name = data.get("name")
        self.members = data.get("members")
        self.image = data.get("image")
        self.english_image = data.get("english_image")
        self.taiwanese_image = data.get("taiwanese_image")
        self.korean_image = data.get("korean_image")
        self.band = data.get("i_band")
        self.tags = data.get("c_tags")
        self.event = data.get("event")
        self.source = data.get("source")
        self.source_link = data.get("source_link")
        self.song = data.get("song")

    def get_comic_members(self):

        d = self._api_get(id=self.cameo, url=self.URL_PARTY+'members/')

        return [PMember(data) for data in d]


class PBackground(PAsset):
    def __init__(self, data):
        super().__init__(data)
        self.name = data.get("name")
        self.members = data.get("members")
        self.image = data.get("image")
        self.english_image = data.get("english_image")
        self.taiwanese_image = data.get("taiwanese_image")
        self.korean_image = data.get("korean_image")
        self.band = data.get("i_band")
        self.tags = data.get("c_tags")
        self.event = data.get("event")
        self.source = data.get("source")
        self.source_link = data.get("source_link")
        self.song = data.get("song")


class PStamp(PAsset):
    def __init__(self, data):
        super().__init__(data)
        self.name = data.get("name")
        self.members = data.get("members")
        self.image = data.get("image")
        self.english_image = data.get("english_image")
        self.taiwanese_image = data.get("taiwanese_image")
        self.korean_image = data.get("korean_image")
        self.band = data.get("i_band")
        self.tags = data.get("c_tags")
        self.event = data.get("event")
        self.source = data.get("source")
        self.source_link = data.get("source_link")
        self.song = data.get("song")

    def get_stamp_members(self):

        d = self._api_get(id=self.members, url=self.URL_PARTY+'members/')

        return [PMember(data) for data in d]


class PTitle(PAsset):
    def __init__(self, data):
        super().__init__(data)
        self.name = data.get("name")
        self.members = data.get("members")
        self.image = data.get("image")
        self.english_image = data.get("english_image")
        self.taiwanese_image = data.get("taiwanese_image")
        self.korean_image = data.get("korean_image")
        self.band = data.get("i_band")
        self.tags = data.get("c_tags")
        self.event = data.get("event")
        self.source = data.get("source")
        self.source_link = data.get("source_link")
        self.song = data.get("song")
        self.value = data.get("value")

    def get_title_event(self):

        d = self._api_get(id=[self.event], url=self.URL_PARTY+'events/')

        return PEvent(d[0])


class PInterface(PAsset):
    def __init__(self, data):
        super().__init__(data)
        self.name = data.get("name")
        self.members = data.get("members")
        self.image = data.get("image")
        self.english_image = data.get("english_image")
        self.taiwanese_image = data.get("taiwanese_image")
        self.korean_image = data.get("korean_image")
        self.band = data.get("i_band")
        self.tags = data.get("c_tags")
        self.event = data.get("event")
        self.source = data.get("source")
        self.source_link = data.get("source_link")
        self.song = data.get("song")


class POfficialArt(PAsset):
    def __init__(self, data):
        super().__init__(data)
        self.name = data.get("name")
        self.members = data.get("members")
        self.image = data.get("image")
        self.english_image = data.get("english_image")
        self.taiwanese_image = data.get("taiwanese_image")
        self.korean_image = data.get("korean_image")
        self.band = data.get("i_band")
        self.tags = data.get("c_tags")
        self.event = data.get("event")
        self.source = data.get("source")
        self.source_link = data.get("source_link")
        self.song = data.get("song")

#############################################################################



import datetime
from .base import BandoriObject

###############################################################################
# Bandori Database models


class DCard(BandoriObject):
    '''
    Represents a bang dream card
    '''
    def __init__(self, data: dict, id_name='cardId', region='en/'):
        super().__init__(data, id_name, region)
        self.character_id = data.get("characterId")
        self.rarity = data.get("rarity")
        self.attribute = data.get("attr")
        self.skill_id = data.get("skillId")
        self.title = data.get("title")
        self.level_limit = data.get("levelLimit")
        self.res_link = (self.URL_GA_RES
                         + '/assets/characters/resourceset/'
                         + data.get("cardRes")
                         + '_rip/'
                         )
        self.image = self.res_link + 'card_normal.png'
        self.image_trained = self.res_link + 'card_after_training.png'
        self.trim = self.res_link + 'trim_normal.png'
        self.trim_trained = self.res_link + 'trim_after_training.png'

        self.live2d_link = (self.URL_GA_RES
                            + '/assets/characters/livesd/'
                            + data.get("live2dRes")
                            + '_rip/'
                            )
        self.chibi = self.live2d_link + 'sdchara.png'

        self.costume_id = data.get("costumeId")
        self.released_at = data.get("releasedAt")
        self.min_stats = data.get("simpleParams").get("min")
        self.max_stats = data.get("simpleParams").get("max")

    def get_skill(self):
        d = self._api_get(url=self.URL_GA + 'skill/' + str(self.skill_id),
                          party=False
                          )

        return DSkill(data=d, region=self.region)


class DSkill(BandoriObject):
    '''
    Represents a Card's skill.
    '''
    def __init__(self, data: dict, id_name='skillId', region='en/'):
        super().__init__(data, id_name, region)
        self.skill_level = data.get("skillLevel")
        self.duration = data.get("duration")
        self.short_description = data.get("simpleDescription")
        self.description = data.get("description")
        self.skill_type = data.get("skillSortType")


class DMember(BandoriObject):
    '''
    Represents a bang dream member.
    Referred to as Character in bandori database.
    '''
    def __init__(self, data: dict, id_name='characterId', region='en/'):
        super().__init__(data, id_name, region)
        self.character_type = data.get("characterType")
        self.band_id = data.get("bandId")
        self.name = data.get("characterName")
        self.ruby = data.get("ruby")

        self.detailed_data = self._api_get(id=[self.id],
                                           url=self.URL_GA+'chara/')


class DDegree(BandoriObject):
    '''
    Represents a ranking from bang dream event.
    '''
    def __init__(self, data: dict, id_name='degreeId', region='en/'):
        super().__init__(data, id_name, region)
        self.seq = data.get("seq")
        self.image_name = data.get("imageName")
        self.degree_rank = data.get("degreeRank")
        self.degree_name = data.get("degreeName")
        self.degree_type = data.get("degreeType")
        self.icon = data.get("iconImageName")
        self.description = data.get("description")


class DComic(BandoriObject):
    '''
    Represents a loading screen koma.
    '''
    def __init__(self, data: dict,
                 id_name='singleFrameCartoonId', region='en/'):
        super().__init__(data, id_name, region)
        self.title = data.get("title")
        self.asset_name = data.get("assetBundleName")
        self.seq = data.get("seq")
        self.subTitle = data.get("subTitle")
        self.asset_link = (self.URL_GA_RES
                           + data.get("assetAddress")
                           )


class DStamp(BandoriObject):
    '''
    Represents a stamp
    '''
    def __init__(self, data, id_name='stampId', region='en/'):
        super().__init__(data, id_name=id_name, region=region)
        self.seq = data.get("seq")
        self.image_link = (self.URL_GA_RES
                           + '/stamp/01_rip/'
                           + data.get("imageName")
                           + '.png'
                           )
        self.type = data.get("stampType")


class DBand(BandoriObject):
    '''
    Represents a bang dream band
    '''
    def __init__(self, data: dict, id_name='bandId', region='en/'):
        super().__init__(data, id_name, region)
        self.name = data.get("bandName")
        self.introduction = data.get("introductions")
        self.type = data.get("bandType")
        self.members = [data.get("leader", -5), data.get("member1", -5),
                        data.get("member2", -5), data.get("member3", -5),
                        data.get("member4", -5)]

        # Note: bands past Roselia have messed up members.

    def get_band_members(self):
        d = self._api_get(id=self.members,
                          url=self.URL_GA+'chara/',
                          party=False)

        return [DMember(data) for data in d]


class DSong(BandoriObject):
    '''
    Represents a playable song in bang dream
    '''
    def __init__(self, data: dict, id_name='musicId', region='en/'):
        super().__init__(data, id_name, region)
        self.title = data.get("title")
        self.bgm = self.URL_GA_RES + data.get("bgmFile", '')
        self.thumb = self.URL_GA_RES + data.get("thumb", '')
        self.jacket = self.URL_GA_RES + data.get("jacket", '')
        self.band_name = data.get("bandName")
        self.band = data.get("bandId")
        self.difficulty = data.get("difficulty")
        self.how_to_get = data.get("howToGet")
        self.achievements = data.get("achievements")
        self.published_at = data.get("publishedAt")
        self.closed_at = data.get("closedAt")

        self.composer = data.get("composer")
        self.lyricist = data.get("lyricist")
        self.arranger = data.get("arranger")


class DGacha(BandoriObject):
    '''
    Represents a gacha in bang dream
    '''
    def __init__(self, data: dict, id_name='gachaId', region='en/'):
        super().__init__(data, id_name, region)
        self.name = data.get("gachaName")
        self.start_date = data.get("publishedAt")
        self.end_date = data.get("closedAt")
        self.description = data.get("description")
        self.rates = data.get("rates")
        self.annotation = data.get("annotation")
        self.gacha_period = data.get("gachaPeriod")
        self.sub_name = data.get("gachaSubName")
        self.type = data.get("gachaType")

    def get_start_date(self):
        return datetime.datetime.fromtimestamp(int(self.start_date) / 1000)

    def get_end_date(self):
        return datetime.datetime.fromtimestamp(int(self.end_date) / 1000)


class DEvent(BandoriObject):
    '''
    Represents an event in bang dream
    '''
    def __init__(self, data, id_name='eventId', region='en/'):
        super().__init__(data, id_name=id_name, region=region)
        self.type = data.get("eventType")
        self.name = data.get("eventName")
        self.asset_name = data.get("assetBundleName")
        self.start_date = data.get("startAt")
        self.end_date = data.get("endAt")
        self.enabled = data.get("enableFlag")
        self.bgm_asset_name = data.get("bgmAssetBundleName")
        self.bgm_file_name = data.get("bgmFileName")
        self.point_rewards = data.get("pointRewards")
        self.rank_rewards = data.get("rankingRewards")
        self.detail = data.get("detail")



from setuptools import setup, find_packages

with open('README.md') as f:
    long_description = f.read()

setup(
  name='pydori',
  packages=find_packages(),
  version='0.5.2',
  license='MIT',
  description="A python wrapper to interact for bandori.party/bandori.ga APIs",
  long_description=long_description,
  long_description_content_type='text/markdown',
  author='William Tang',
  url='https://github.com/WiIIiamTang/pydori',
  keywords=[
    'API', 'Bandori', 'Bang Dream', 'rythm game'
    ],
  install_requires=[
    'requests'
    ],
  classifiers=[
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    "Operating System :: OS Independent"
    ],
)
